// Timer fr Kaffeemhle mit ATTINY84A

#define F_CPU 9830400L

#include <stdint.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include <util/delay.h>
#include "tm1637.h"

#define SCHUETZ_AN  PORTB |=  (1<<PB2)
#define SCHUETZ_AUS PORTB &= ~(1<<PB2)

#define MUEHLE_AN   PORTA |=  (1<<PA7)
#define MUEHLE_AUS  PORTA &= ~(1<<PA7)

#define KEY_DDR         DDRA
#define KEY_PORT        PORTA
#define KEY_PIN         PINA
#define PLUS            0
#define MINUS           1
#define ENTER           2
#define ALL_KEYS        (1<<PLUS | 1<<MINUS | 1<<ENTER)

#define REPEAT_MASK     (1<<PLUS | 1<<MINUS  | 1<<ENTER)   // repeat: plus, minus, enter
#define REPEAT_START    50                                 // after 500ms
#define REPEAT_NEXT     10                                 // every 100ms

volatile uint16_t mahlzeit;
volatile uint8_t  timer_aktiv=0;
volatile uint16_t countdown;
uint16_t laufzeit EEMEM;

volatile uint8_t key_state; // debounced and inverted key state:
                            // bit = 1: key pressed
volatile uint8_t key_press; // key press detect
volatile uint8_t key_rpt;   // key long press and repeat

// Funktionsprototypen
void display ( uint16_t );
void timer0_init ( void );
void timer1_init ( void );
uint8_t get_key_press( uint8_t );
uint8_t get_key_rpt( uint8_t );
uint8_t get_key_state( uint8_t );
uint8_t get_key_short( uint8_t );
uint8_t get_key_long( uint8_t );

int main ( void )
{
    PORTA |= (1<<PA0) | (1<<PA1) | (1<<PA2);    // Pullups fr drei Taster an
    DDRA  |= (1<<PA4) | (1<<PA5) | (1<<PA7);    // Display und Mhle als Ausgang
    DDRB  |= (1<<PB2);                          // Schtz als Ausgang

    uint8_t x=0;
    
    _delay_ms(1000); // Schtz nicht sofort nach Druck auf den Taster einschalten
    SCHUETZ_AN;

    while(x!=3)      // Warten, bis der Taster losgelassen wurde
    {
        x=0;
        _delay_ms(100);
        if( PINA & (1<<ENTER) ) x++;
        _delay_ms(100);
        if( PINA & (1<<ENTER) ) x++;
        _delay_ms(100);
        if( PINA & (1<<ENTER) ) x++;
    }
    
    timer0_init();
    timer1_init();
    sei();
    
    mahlzeit=eeprom_read_word(&laufzeit);
    if(mahlzeit>9999) mahlzeit=100;
    if(mahlzeit==0) mahlzeit=100;
    
    TM1637_init(1/*enable*/, 5/*brightness*/);
    TM1637_display_colon(1);
    display(mahlzeit);

    while(1)
    {
        if( get_key_short( 1<<ENTER ) )
        {
            if(timer_aktiv==0)
            {
                eeprom_update_word(&laufzeit, mahlzeit);
                countdown=mahlzeit;
                TCNT1=0;
                timer_aktiv=1;
                MUEHLE_AN;
            }
            else
            {
                timer_aktiv=0;
                MUEHLE_AUS;
                display(mahlzeit);
            }
        }
        if( get_key_long( 1<<ENTER ) )
        {
            SCHUETZ_AUS;
        }

        if( get_key_press( 1<<PLUS ) || get_key_rpt( 1<<PLUS ) )
        {
            if(timer_aktiv==0)
            {
                if(mahlzeit < 9998)
                {
                    mahlzeit++;
                    display(mahlzeit);
                }
            }
        }

        if( get_key_press( 1<<MINUS ) || get_key_rpt( 1<<MINUS ) )
        {
            if(timer_aktiv==0)
            {
                if(mahlzeit > 0)
                {
                    mahlzeit--;
                    display(mahlzeit);
                }
            }
        }
        
    }
}



void display ( uint16_t zahl)
{
    // Knnte man noch ndern, um fhrende Nullen nicht anzuzeigen.
	TM1637_display_digit(0, zahl / 1000);
    TM1637_display_digit(1, zahl % 1000 / 100);
    TM1637_display_digit(2, zahl % 100 / 10);
    TM1637_display_digit(3, zahl % 10);
}


void timer0_init (void) // ISR alle 10 ms
{
    TCCR0A |= (1<<WGM01); // CTC
    
    //    TCCR0B |= (1<<CS00);                // kein Prescaler
    //    TCCR0B |= (1<<CS01);                // Prescaler 8
    //    TCCR0B |= (1<<CS01) | (1<<CS00);    // Prescaler 64
    //    TCCR0B |= (1<<CS02);                // Prescaler 256
    TCCR0B |= (1<<CS02) | (1<<CS00);    // Prescaler 1024

    OCR0A = 95;
    
    TIMSK0 |= (1<<OCIE0A);                    // Compare Match Interrupt enable
}

void timer1_init (void) // ISR alle 100 ms
{
    TCCR1B |= (1<<WGM12); // CTC
    
    //    TCCR1B |= (1<<CS10);                // kein Prescaler
    //    TCCR1B |= (1<<CS11);                // Prescaler 8
    //    TCCR1B |= (1<<CS11) | (1<<CS10);    // Prescaler 64
    //    TCCR1B |= (1<<CS12);                // Prescaler 256
    TCCR1B |= (1<<CS12) | (1<<CS10);    // Prescaler 1024

    OCR1A = 959;
    
    TIMSK1 |= (1<<OCIE1A);                // Compare Match Interrupt enable
}

ISR ( TIM1_COMPA_vect )   // alle 100 Millisekunden
{
    static uint8_t zehntel;
    static uint8_t sekunden;
    static uint8_t minuten;
    
    zehntel++;
    if(zehntel==10)
    {
        zehntel=0;
        sekunden++;
        if(sekunden==60)
        {
            sekunden=0;
            minuten++;
            if(minuten==30)
            {
                minuten=0;
                SCHUETZ_AUS;
            }
        }
    }
    
    if(timer_aktiv==1)
    {
        minuten=0;
        sekunden=0;
        
        countdown--;
        if(countdown==0)
        {
            timer_aktiv=0;
            MUEHLE_AUS;
            countdown=mahlzeit;
        }
        display(countdown);
    }
}


ISR( TIM0_COMPA_vect )                              // every 10ms
{
    static uint8_t ct0 = 0xFF, ct1 = 0xFF, rpt;
    uint8_t i;

    i = key_state ^ ~KEY_PIN;                       // key changed ?
    ct0 = ~( ct0 & i );                             // reset or count ct0
    ct1 = ct0 ^ (ct1 & i);                          // reset or count ct1
    i &= ct0 & ct1;                                 // count until roll over ?
    key_state ^= i;                                 // then toggle debounced state
    key_press |= key_state & i;                     // 0->1: key press detect

    if( (key_state & REPEAT_MASK) == 0 )            // check repeat function
    rpt = REPEAT_START;                             // start delay
    if( --rpt == 0 ){
        rpt = REPEAT_NEXT;                          // repeat delay
        key_rpt |= key_state & REPEAT_MASK;
    }
}

///////////////////////////////////////////////////////////////////
//
// check if a key has been pressed. Each pressed key is reported
// only once
//
uint8_t get_key_press( uint8_t key_mask )
{
    cli();                                          // read and clear atomic !
    key_mask &= key_press;                          // read key(s)
    key_press ^= key_mask;                          // clear key(s)
    sei();
    return key_mask;
}

///////////////////////////////////////////////////////////////////
//
// check if a key has been pressed long enough such that the
// key repeat functionality kicks in. After a small setup delay
// the key is reported being pressed in subsequent calls
// to this function. This simulates the user repeatedly
// pressing and releasing the key.
//
uint8_t get_key_rpt( uint8_t key_mask )
{
    cli();                                          // read and clear atomic !
    key_mask &= key_rpt;                            // read key(s)
    key_rpt ^= key_mask;                            // clear key(s)
    sei();
    return key_mask;
}

///////////////////////////////////////////////////////////////////
//
// check if a key is pressed right now
//
uint8_t get_key_state( uint8_t key_mask )
{
    key_mask &= key_state;
    return key_mask;
}

///////////////////////////////////////////////////////////////////
//
uint8_t get_key_short( uint8_t key_mask )
{
    cli();                                          // read key state and key press atomic !
    return get_key_press( ~key_state & key_mask );
}

///////////////////////////////////////////////////////////////////
//
uint8_t get_key_long( uint8_t key_mask )
{
    return get_key_press( get_key_rpt( key_mask ));
}

